import { ComponentContainer, ResolvedLayoutConfig } from '@genesis-community/golden-layout';
import { componentType, instanceContainer } from './constants';
/**
 * Definition of a custom button which will be added to all layout items.
 * @remarks
 * `svg` - string of the SVG to use for the button. Needs to be in the format `data:image/svg+xml;base64,<<base64 encoded definition>>`.
 * `onClick` - function which will be called when the button is clicked. The clicked button and the contained element associated with the clicked button will be passed to the function.
 * @public
 */
export type CustomButton = {
    svg: string;
    onClick: (button: HTMLElement, element: HTMLElement) => void;
};
/**
 * @public
 * Versioned layout config objects. `v` is the version and `c` contains the layout config.
 * @remarks
 * Versioning the layout config in this way allows changes to the config schema while
 * remaining backwards compatible with any existing client/user configurations. When loading an
 * older version of the configurations the version can be used to lookup and apply transformer functions
 * to get the newest version of the config from it.
 *
 * As more versions of configurations are added this type will have extra union types as part of SerialisedLayout.
 */
export type SerialisedLayout = {
    v: '1';
    c: ResolvedLayoutConfig;
};
/**
 * Interface to implement on an item which is a component of the layout and you wish to serialise state with. This is saved separately for each instance of the component, which allows you to restore multiple instances of the same component with different state.
 * @typeParam T - the type of the state object you wish to serialise with the component.
 * @remarks
 * When the layout is saved either via the autosave functionality or manually calling {@link FoundationLayout.getLayout}, all contained components will be requested to provide state if they wish.
 *
 * Any state which is provided will be saved as part of the layout config and will be passed back to the component when the layout is loaded with {@link FoundationLayout.loadLayout} (or via autoloading). The state will be applied to the component via `applyState`, this may be before or after the component has been placed on the DOM, so you'll need to handle either scenario. The state is `null` when the instance is first created.
 *
 * See the written documentation for some error scenarios to consider about when implementing this interface.
 *
 * @example
 * ```
 * type ComponentState = {
 *   foo: string;
 * }
 * \@customElement({ name: 'my-component' })
 * export class MyComponent extends FASTElement implements LayoutComponentWithState<ComponentState> {
 *   \@observable foo: string;
 *   private fooCache: ComponentState | null;
 *
 *   getCurrentState(): ComponentState {
 *     if (!this.foo) return null;
 *     return {
 *       foo: this.foo;
 *     }
 *   }
 *
 *   applyState(state: ComponentState | null) {
 *     this.fooCache = state;
 *   }
 *
 *   connectedCallback() {
 *     // do other required setup
 *     if (this.fooCache) {
 *       this.foo = this.fooCache.foo;
 *     }
 *   }
 * }
 * ```
 *
 * @example
 * If you are using the autosave functionality you should inform the layout system when you update the state of a component, otherwise the state will only be updated when the user performs an action such as resizing an item. Use the {@link LayoutReceiveEvents} `autosave` event.
 * ```
 * // Same component as above
 * export class MyComponent extends FASTElement implements LayoutComponentWithState<ComponentState> {
 *  // can use xChanged pattern as `foo` was declared observable
 *  fooChanged() {
 *    this.$emit(LayoutReceiveEvents.autosave);
 *  }
 * }
 * ```
 * @public
 * */
export interface LayoutComponentWithState<T> {
    /**
     * Provide the state you wish to save. It is recommended if the component which implements this interface has not fully initialised at the point this is called that you return `null` as the state, following the pattern of `null` being set as the initial state.
     */
    getCurrentState(): T;
    /**
     * Handle any state that has been saved previously for this instance of this component. It is not deterministic to know whether this is called before the component is appended to the DOM.
     */
    applyState(state: T | null): void;
}
/**
 * @public
 * The parameters that can be set on a new item when being added by the {@link FoundationLayout.addItem} API
 *
 * @remarks
 * `elements` - array of Elements that are the content of the new item
 * `title` - optional string which is used as the new title of the tab (defaults to be the registered name)
 * `closable` - optional boolean which allows the item to be closed with the X button if set and true
 * `size` - optional string describing the size of the new item (see the written documentation for more info)
 * `registration` - optional string configuring the registration ID of the new item (defaults to sequential IDs)
 */
export interface RegisteredElementConfig {
    registration: string;
    title?: string;
    closable?: boolean;
    size?: string;
}
/**
 * @public
 * Where to and how to add the new item(s) into the layout when using the {@link FoundationLayout.addItem} API.
 * @remarks
 * `area` dictates what part of the layout the new item(s) will be added to.
 * `multiple` is optional and is defaulted to `type=horizontal` and `size=50%` if not defined. `multiple` config only affects the placement if multiple items are specified to be added.
 * `multiple.type` sets the group type of the new items, whether they are horizontal, vertical, or in tabs.
 * `multiple.size` - optional string describing the size of the new item (see the written documentation for more info).
 * `multiple.size` specifies the size of the new group to be added and then the `size` config of the new items are the sizes relative to this new multiple group. If you only specify one new item then the `size` of that will be used as the size and will override `multiple.size`.
 */
export type Placement = {
    area: 'top' | 'left' | 'bottom' | 'right';
    multiple?: {
        type: LayoutRegionType;
        size?: string;
    };
};
/** @internal */
export type InstanceContainer = {
    container: ComponentContainer;
    instance: string;
};
/** @internal */
export type LayoutComponentItem<T> = Element & Partial<LayoutComponentWithState<T>> & {
    [instanceContainer]?: InstanceContainer;
};
/** @internal */
export interface RegistrationConfig {
    elements: Element[];
    id?: string;
}
/** @internal */
export declare const layoutRegionValue: readonly ["horizontal", "vertical", "tabs"];
/**
 * @public
 * Union type describing the three different types of region splits.
 * Set on the `type` attribute on {@link FoundationLayoutRegion}.
 */
export type LayoutRegionType = (typeof layoutRegionValue)[number];
/** @internal */
export type ComponentTypes = LayoutRegionType | 'root' | 'item';
/** @internal */
export interface LayoutComponent {
    [componentType]: ComponentTypes;
    cacheElementsAndRegister(config: RegistrationConfig): string;
    requestLayoutReload(): void;
}
/** @internal */
export type AutosavedLayouts = {
    [x: string]: string;
};
//# sourceMappingURL=types.d.ts.map